/*
 * Copyright (c) 2023, salesforce.com, inc.
 * All rights reserved.
 * Licensed under the BSD 3-Clause license.
 * For full license text, see LICENSE.txt file in the repo root or https://opensource.org/licenses/BSD-3-Clause
 */
import { readdirSync } from 'node:fs';
import { readFile } from 'node:fs/promises';
import { join, resolve } from 'node:path';
import { fileURLToPath } from 'node:url';
import { SfError, SfProject, Messages } from '@salesforce/core';
import { ComponentSetBuilder, MetadataConverter, RegistryAccess, presetMap, } from '@salesforce/source-deploy-retrieve';
import { isString } from '@salesforce/ts-types';
// TODO: there could be a cleaner way to read this
const PRESET_DIR = fileURLToPath(join(import.meta.resolve('@salesforce/source-deploy-retrieve'), '..', 'registry', 'presets'));
export const PRESETS_PROP = 'sourceBehaviorOptions';
export const PRESET_CHOICES = [...presetMap.keys()];
export const TMP_DIR = process.env.SF_MDAPI_TEMP_DIR ?? 'decompositionConverterTempDir';
export const DRY_RUN_DIR = 'DRY-RUN-RESULTS';
/** returns packageDirectories and ComponentsSets where there is metadata of the type we'll change the behavior for */
export const getPackageDirectoriesForPreset = async (project, preset) => {
    const projectDir = project.getPath();
    const messages = loadMessages();
    const output = (await Promise.all(project
        .getPackageDirectories()
        .map((pd) => pd.path)
        .map(componentSetFromPackageDirectory(projectDir)(await getTypesFromPreset(preset))))).filter(componentSetIsNonEmpty);
    if (output.length === 0) {
        messages.createError('error.noTargetTypes', [preset]);
    }
    return output;
};
/** converts the composed metadata to mdapi format in a temp dir */
export const convertToMdapi = async (packageDirsWithDecomposable) => (await Promise.all(packageDirsWithDecomposable.map(async (pd) => {
    // convert to the mdapi targetDir
    await new MetadataConverter().convert(pd.cs, 'metadata', {
        type: 'directory',
        outputDirectory: join(TMP_DIR, pd.packageDirPath),
        genUniqueDir: false,
    });
    return getComponentSetFiles(pd.cs);
})))
    .flat()
    .map((f) => resolve(f));
/** get the LOCAL project json, throws if not present OR the preset already exists */
export const getValidatedProjectJson = (preset, project) => {
    const projectJson = project.getSfProjectJson(false);
    if (projectJson.get(PRESETS_PROP)?.includes(preset)) {
        throw SfError.create({
            name: 'sourceBehaviorOptionAlreadyExists',
            message: `sourceBehaviorOption ${preset} already exists in sfdx-project.json`,
        });
    }
    return projectJson;
};
/** converts the temporary mdapi back to source, return a list of the created files */
export const convertBackToSource = async ({ packageDirsWithPreset, projectDir, dryRun, }) => [
    ...new Set((await convertToSource({
        packageDirsWithPreset,
        projectDir,
        dryRunDir: dryRun ? DRY_RUN_DIR : undefined,
    }))
        .flatMap((cr) => cr.converted ?? [])
        // we can't use walkContent because there's a conditional inside it
        .flatMap(getSourceComponentFiles)
        .filter(isString)),
];
const getSourceComponentFiles = (c) => [c.xml, ...(c.content ? fullPathsFromDir(c.content) : [])].filter(isString);
const fullPathsFromDir = (dir) => readdirSync(dir, { withFileTypes: true }).map((d) => join(d.path, d.name));
/** build a component set from the original project for each pkgDir */
const componentSetFromPackageDirectory = (projectDir) => (metadataEntries) => async (packageDir) => ({
    packageDirPath: packageDir,
    cs: await ComponentSetBuilder.build({
        metadata: {
            metadataEntries,
            directoryPaths: [packageDir],
        },
        projectDir,
    }),
});
const convertToSource = async ({ packageDirsWithPreset, projectDir, dryRunDir, }) => {
    // mdapi=>source convert the target dir back to the project
    // it's a new converter because the project has changed and it should reload the project's registry.
    SfProject.clearInstances(); // break the singleton so SDR will re-read to get the new preset
    const converter = new MetadataConverter(new RegistryAccess(undefined, projectDir));
    return Promise.all(packageDirsWithPreset.map(async (pd) => converter.convert(
    // componentSet based on each mdapi folder
    await ComponentSetBuilder.build({ sourcepath: [join(TMP_DIR, pd.packageDirPath)], projectDir }), 'source', dryRunDir
        ? // dryRun outputs to a dir outside the real packageDirs folder to avoid changing real stuff
            {
                type: 'directory',
                outputDirectory: join(projectDir, dryRunDir),
                packageName: pd.packageDirPath,
                genUniqueDir: false,
            }
        : {
            type: 'merge',
            mergeWith: (await ComponentSetBuilder.build({
                sourcepath: [pd.packageDirPath],
                projectDir,
            })).getSourceComponents(),
            defaultDirectory: join(projectDir, pd.packageDirPath),
        })));
};
export const getTypesFromPreset = async (preset) => Object.values(JSON.parse(await readFile(join(PRESET_DIR, `${preset}.json`), 'utf-8')).types).map((t) => t.name);
const getComponentSetFiles = (cs) => cs
    .getSourceComponents()
    .toArray()
    .flatMap((c) => [c.xml, ...c.walkContent()])
    .filter(isString);
const loadMessages = () => {
    Messages.importMessagesDirectoryFromMetaUrl(import.meta.url);
    return Messages.loadMessages('@salesforce/plugin-deploy-retrieve', 'convert.source-behavior');
};
const componentSetIsNonEmpty = (i) => i.cs.size > 0;
//# sourceMappingURL=convertBehavior.js.map